/* Thermometre, baromètre, Altimetre, Inclinomètre, acceleromètre
 *  avec module Arduino Nano, Accéleromètre GY88 et LCD 2lignes x 8Caractères.
 *  B.LECLERC 08/17
 */
#include <LiquidCrystal.h> //Library du LCD
#include <Adafruit_BMP085.h> //Library de thermomètre et baromète
#include <MPU6050.h> //Library de l'accéléromètre et gyroscope

Adafruit_BMP085 bmp; //Déclaration Data du baromètre
MPU6050 accelgyro; //Déclaration Data acceleromètre & gyroscope

int16_t ax, ay, az; //Variables accèleromètre
int16_t gx, gy, gz; //Variables accèleromètre


int mx; //Memorisation de la valeur accéleration X
int my; //Memorisation de la valeur accéleration y
int mz; //Memorisation de la valeur accéleration z
int dx; //Memorisation du delta entre ancienne mesure et nouvelle mesure d'accéleration X
int dy; //Memorisation du delta entre ancienne mesure et nouvelle mesure d'accéleration Y
int dz; //Memorisation du delta entre ancienne mesure et nouvelle mesure d'accéleration Z
int n; //Compteur de boucle
int mesure = 13; //Déclaration de la Led rouge de l'arduino pendant la mesure 
int led = 6; // LED du rétro-éclairage du LCD connectée à la sortie digitale pin 6
int button = 7; // Button conncté à l'entrée digitale pin 7

//Déclarations et fonctions des pin de connection du LCD
const int rs = 12, en = 11, d4 = 5, d5 = 4, d6 = 3, d7 = 2;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

void setup() {
  pinMode(led, OUTPUT); //Rétro-éclairage en sortie
  pinMode(mesure, OUTPUT); //Led de mesure en sortie 
  pinMode (button, INPUT); //Bouton en entrée dans pull up par arduino

  //Initialisation de l'accéléromètre
     accelgyro.initialize();
     
  // Définiles colonnes et rangées du LCD  
    lcd.begin(8, 2);
  // Allume le rétro-éclairage
    digitalWrite(led, HIGH);   
  // Message d'accueil sur la 1°ligne
    lcd.print(" Nanard ");
  // positionne le curseur colonne 0, ligne 1
    lcd.setCursor(1, 1);
  // Ecrit le message sur la ligne du bas du LCD  
    lcd.print("08/17");
    delay (1000);  //Pause de 1"

  
  //Initialisation du BMP085 pour toujours
    if (!bmp.begin()) {
  while (1) {}
  }
}


void loop() {
  digitalWrite(led, HIGH);   // Allume le rétro-éclairage
/* Aquisition et affichage de la température ambiante par le module BMP085
 *  La première mesure est la plus vraie, car le module s'échauffe et la température 
 *  à tendance à s'élever...
 */
  Temp:
    lcd.setCursor(0,0);// positionne le curseur colonne 0, ligne 0
      lcd.print("Temp =   ");// Message d'entête sur la 1°ligne
        lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
          digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
        lcd.print(bmp.readTemperature(),1);// Mesure de la température et affichage à 1 décimale après la virgule
      digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
    lcd.print(" *C  ");//Affichage de l'unité (* a la place de ° car non reconnu)
  delay (2000);//Pause de 2"
  if (digitalRead(button)) { // lit l'état du bouton
    goto Press; // Et passe au menu suivant si actif 
  }
  goto Temp; //On boucle avec le début du menu si pas d'action sur le bouton

  
/* Aquisition et affichage de la Pression atmosphérique par le module BMP085
 *  Le baromètre affiche la pression en Pascal (Pa)! une haute pression garanti du beau temps
 *  Et une basse pression, une dégradation en cours ...
 */
Press:
  lcd.setCursor(0,0);// positionne le curseur colonne 0, ligne 0
    lcd.print("P.Atmo. ");// Message d'entête sur la 1°ligne
      lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
        digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure 
      lcd.print(bmp.readPressure());// Mesure de la pression atmosphèrique et affichage
    digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
  lcd.print(" Pa   ");//Affichage de l'unité 
delay (2000);//Pause de 2"
  if (digitalRead(button)) { // lit l'état du bouton
    goto Alti; // Et passe au menu suivant si actif
  }
goto Press; //On boucle avec le début du menu si pas d'action sur le bouton


/* Aquisition et affichage de l'altitude du lieu par le module BMP085
 *  L'Altimètre affiche la hauteur du lieu au dessus du niveau de la mer!
 *  L'altitude est affichée en mètres ...
 */
Alti:
  lcd.setCursor(0,0);// positionne le curseur colonne 0, ligne 0
    lcd.print("Altitude");// Message d'entête sur la 1°ligne
      lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
        digitalWrite(mesure,HIGH); // Allume la led 13 pour marquer la mesure
      lcd.print(bmp.readAltitude(),0);// Mesure de l'Altitude et affichage
    digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
  lcd.print(" Mtr   ");//Affichage de l'unité 
delay (2000);//Pause de 2"
  if (digitalRead(button)) { // lit l'état du bouton
      lcd.clear();// effacement du LCD
    lcd.print("Horizon.: ");// Message d'entête sur la 1°ligne
    goto Hori_x;  // Et passe au menu suivant si actif
  }
goto Alti;//On boucle avec le début du menu si pas d'action sur le bouton


/* Aquisition et affichage d'une variation d'horizon dans le sens de la longueur
 * La variation est notée en positif ou négatif suivant le sens
 */
Hori_x:
      digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
        accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
      digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
     lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
   lcd.print(ax/185);lcd.print(" x   ");//Affichage de la mesure et de l'axe
  delay (500); //Pause de 1/2"
if (digitalRead(button)) { // lit l'état du bouton
  goto Hori_y; // Et passe au menu suivant si actif 
  }
goto Hori_x;//On boucle avec le début du menu si pas d'action sur le bouton


/* Aquisition et affichage variation d'horizon dans le sens de la largeur du boitier
 * La variation est notée en positif ou négatif suivant l'inclinaison par rapport à l'horizon 
 * Une grande précision mécanique assure des mesures plus précises ...
 */
Hori_y:
      digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
        accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
      digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
    lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
  lcd.print(ay/184);lcd.print(" y   ");//Affichage de la mesure et de l'axe
  delay (500); //Pause de 1/2"
if (digitalRead(button)) { // lit l'état du bouton
  goto Hori_z;  // Et passe au menu suivant si actif
  }
goto Hori_y;//On boucle avec le début du menu si pas d'action sur le bouton


/* Aquisition et affichage variation d'inclinaison dans le sens de la verticalité du boitier
 * La variation est notée en positif ou négatif suivant l'inclinaison par rapport à l'axe vertical 
 * Une grande précision mécanique assure des mesures plus précises ...
 */
Hori_z:
      digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
        accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
      digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
    lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
  lcd.print(az/167);lcd.print(" z    ");//Affichage de la mesure et de l'axe
  delay (500); //Pause de 1/2"
if (digitalRead(button)) { // lit l'état du bouton
  lcd.clear();// effacement du LCD
  lcd.print("Accel.: ");// Message d'entête sur la 1°ligne
    goto Accel_x;  // Et passe au menu suivant si actif
  }
goto Hori_z;//On boucle avec le début du menu si pas d'action sur le bouton



/* Aquisition et affichage variation Gyroscopyque dans l'axe de la longueur du boitier
 * La variation est notée en positif ou négatif
 * La mesure est effectuée deux fois par seconde ...
 */
Accel_x:
    lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
      digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
        accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
      digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
    lcd.print(gx);lcd.print(" x    ");//Affichage de la mesure et de l'axe
  delay (500); //Pause de 1/2"
if (digitalRead(button)) { // lit l'état du bouton
  goto Accel_y;  // Et passe au menu suivant si actif
}
goto Accel_x;//On boucle avec le début du menu si pas d'action sur le bouton


/* Aquisition et affichage variation Gyroscopyque dans l'axe de la largeur du boitier
 * La variation est notée en positif ou négatif
 * La mesure est effectuée deux fois par seconde ...
 */
Accel_y:
    lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
      digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
        accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
      digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
    lcd.print(gy);lcd.print(" y    ");//Affichage de la mesure
  delay (500); //Pause de 1/2"
if (digitalRead(button)) { // lit l'état du bouton
  goto Accel_z;  // Et passe au menu suivant si actif
}
goto Accel_y;//On boucle avec le début du menu si pas d'action sur le bouton



/* Aquisition et affichage variation Gyroscopyque dans l'axe de la hauteur du boitier
 * La variation est notée en positif ou négatif
 * La mesure est effectuée deux fois par seconde ...
 */
Accel_z:
    lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
      digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
        accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
      digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
    lcd.print(gz);lcd.print(" z    ");//Affichage de la mesure et de l'axe
  delay (500); //Pause de 1/2"
if (digitalRead(button)) { // lit l'état du bouton
  lcd.clear();// effacement du LCD
    lcd.print("Delta:  ");// Message d'entête sur la 1°ligne
  goto Delta_x; // Et passe au menu suivant si actif 
}
goto Accel_z;//On boucle avec le début du menu si pas d'action sur le bouton


/* Aquisition et affichage de la différence de valeur entre la dernière mesure 
 *  et la nouvelle dans l'axe de la longueur du boitier
 * La variation est notée en positif ou négatif
 * La mesure est effectuée une fois par seconde ...
 */
Delta_x:
n = n + 1; //Incrémentation du nombre de boucle 
  mx = gx;//Recopie de la valeur en vue du stocage
    digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
      accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
        digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
          dx = gx - mx;// Soustraction de la nouvelle valeur de l'ancienne valeur
        if ((dx > 30)or (dx < -30)) { // Affichage si et seullement si la variation est suffisante
      lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
    lcd.print(dx);lcd.print(" x    ");//Affichage de la différence 
  }
delay (500); //Pause de 1/2"
  if (digitalRead(button)) { // lit l'état du bouton
    n = 0; //RaZ de la boucle si bouton.... 
    goto Delta_y; // Et passe au menu suivant si actif 
  }
  if (n > 100) {//Si la boucle atteind les 100 on met en veille
    goto Veille;
  }
  if ((gx > 2000) or (gy > 700) or (gz > 800) ) { (n = 0); } //RaZ de la boucle si mouvement ! 
goto Delta_x;//On boucle avec le début du menu si pas d'action sur le bouton



/* Aquisition et affichage de la différence de valeur entre la dernière mesure 
 *  et la nouvelle dans l'axe de la largeur du boitier
 * La variation est notée en positif ou négatif
 * La mesure est effectuée deux fois par seconde ...
 */
 Delta_y:
 n = n + 1; //Incrémentation du nombre de boucle 
  my = gy;//Recopie de la valeur en vue du stocage
    digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
      accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
        digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
          dy = gy - my;// Soustraction de la nouvelle valeur de l'ancienne valeur
        if ((dy > 30)or (dy < -30)) {// Affichage si et seullement si la variation est suffisante
      lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
    lcd.print(dy);lcd.print(" y    ");//Affichage de la différence
  }
delay (500); //Pause de 1/2"
   if (digitalRead(button)) { // lit l'état du bouton
    n = 0; //RaZ de la boucle si bouton .... 
    goto Delta_z;  // Et passe au menu suivant si actif
  }
  if (n > 100) {//Si la boucle atteind les 100 on met en veille
    goto Veille;
  }
  if ((gx > 2000) or (gy > 700) or (gz > 800) ) { (n = 0); }  //RaZ de la boucle si  mouvement  ! 
goto Delta_y;//On boucle avec le début du menu si pas d'action sur le bouton



/* Aquisition et affichage de la différence de valeur entre la dernière mesure 
 *  et la nouvelle dans l'axe de la hauteur du boitier
 * La variation est notée en positif ou négatif
 * La mesure est effectuée deux fois par seconde ...
 */
Delta_z:
n = n + 1; //Incrémentation du nombre de boucle 
  mz = gz;//Recopie de la valeur en vue du stocage
    digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
      accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
        digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
          dz = gz - mz;// Soustraction de la nouvelle valeur de l'ancienne valeur
        if ((dz > 30)or (dz < -30)) {// Affichage si et seullement si la variation est suffisante
      lcd.setCursor(0, 1);// positionne le curseur colonne 0, ligne 1
    lcd.print(dz);lcd.print(" z    ");//Affichage de la différence
  }
delay (500); //Pause de 1/2"
  if (digitalRead(button)) { // lit l'état du bouton
    n = 0; //RaZ de la boucle si bouton.... 
    goto Temp;  // Et passe au menu suivant si actif
  }
  if (n > 100) { //Si la boucle atteind les 100 on met en veille
    goto Veille;
  }
if ((gx > 2000) or (gy > 700) or (gz > 800) ) { (n = 0); }  //RaZ de la boucle si mouvement....  ! 
goto Delta_z;//On boucle avec le début du menu si pas d'action sur le bouton
   

/* Extinction du rétro éclairage pour réduir la consommation 
 *  et retour au début si le bouton est sollicité
 *  ou si un mouvement est reçu.
 */
Veille:
  digitalWrite(led, LOW);   // éteind le rétro-éclairage .
    delay (500);//Pause de 1/2"
        digitalWrite(mesure,HIGH);// Allume la led 13 pour marquer la mesure
          accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);// Aquisition des 6 valeurs
        digitalWrite(mesure,LOW);// Eteind la led 13 pour marquer la fin de mesure
if (((digitalRead(button)) or (gx > 2000) or (gy > 700) or (gz > 800) )) { // lit l'état du bouton et du mouvement
  digitalWrite(led, HIGH);   // sets the LED on
    n = 0; //RaZ de la boucle si bouton ou mouvement....   
  goto Temp;  // Et passe au début suivant si actif
}
  delay (2000);//Pause de 2" 
  goto Veille;//On boucle avec le début du menu si pas d'action sur le bouton
}


